<?php

	# Some credits, which show up on the wiki page Special:Version
	#
	$wgExtensionCredits['parserhook'][] = array(
	  'name'		=> 'Clustering Demo',
	  'version'		=> '1.0',
	  'description'	=> 'Applet which demonstrates K-Means and X-Means clustering',
	  'author'		=> 'R.E.Scarberry'
	  );

	# This extension function enables the tag extension <clustering .../> and
	# <clustering ...></clustering>
    $wgExtensionFunctions[] = 'clusteringDemoSetup';

	# These lines enable the parser function "#clustering"
    $wgExtensionFunctions[] = 'clusteringDemoFunctionSetup';

	# Also register a hook to initialize the magic word.
	$wgHooks['LanguageGetMagic'][] = 'clusteringDemoFunctionMagic';

    $clusteringDemoAppletEmbedded = false;

	# The callback to associate the rendering function with the tag extension "clustering"
    function clusteringDemoSetup() {
        global $wgParser;
        # Set the rendering function for the tag 'clustering' to the function clusteringDemoRender.
        $wgParser->setHook('clustering', 'clusteringDemoRender');
        # Set the rendering function for the tag 'clustering_data' to a function that returns nothing.  The applet
        # uses this data, but it should not affect what the user sees on the rendered page.
        $wgParser->setHook('clustering_data', 'blankRender');
    }

	# These next two functions are callbacks to set up the parser function "#clustering"
    function clusteringDemoFunctionSetup() {
    	global $wgParser;
    	$wgParser->setFunctionHook('clustering', 'clusteringDemoFunctionRender');
    }
   	function clusteringDemoFunctionMagic(&$magicWords, $langCode) {
		$magicWords['clustering'] = array(0, 'clustering');
		return true;
	}

	# The rendering function for the tag extension <clustering>
	# ($args is an associative array containing the attributes.)
	#
	function clusteringDemoRender($input, $args, $parser) {
    	global $wgOut, $wgScriptPath, $wgCookiePrefix, $clusteringDemoAppletEmbedded;

        if ($clusteringDemoAppletEmbedded) {
            return "<em>Only one clustering applet per page!</em>";
        }

        $clusteringDemoAppletEmbedded = true;

        # Adds the stylesheet link to the head section of the page.   The stylesheet contains the styles for the divs below, including
        # the applet width and height and the resize handle attributes.
        #
        $wgOut->addScript('<link rel="stylesheet" type="text/css" href="' .
            $wgScriptPath . '/extensions/clustering_demo/css/clustering_demo.css" />');

        # Adds the prototype and scriptaculous javascript libraries, which are needed by clustering_demo.js.
        #
        $wgOut->addScript('<script src="' . $wgScriptPath .
            '/extensions/clustering_demo/js/lib/prototype.js" type="text/javascript"></script>');
        $wgOut->addScript('<script src="' . $wgScriptPath .
            '/extensions/clustering_demo/js/lib/scriptaculous.js?load=effects,dragdrop" type="text/javascript"></script>');

        # Adds the javascript for this extension.  Its purpose is to set up and control the resize handles.
        #
        $wgOut->addScript('<script src="' . $wgScriptPath .
            '/extensions/clustering_demo/js/clustering_demo.js" type="text/javascript"></script>');

        $output =
            "<div id='clustering_demo_div'>" .
	        "<applet name='clustering_demo_applet' id='clustering_demo_applet' code='clustering.DemoApplet' archive='clustering_demo.jar,commons-logging-1.1.1.jar' " .
	         "codebase='/mediawiki/extensions/clustering_demo'>\n";

        # Flags set to true if $args contains the extension tag used on the edit tab as well as the extension type -- tag, function, template
        # These parameters are passed to the applet so it can figure out how to find its tag extension, parser function, or template expression in
        # the edit text of its web page.
        $gotExtensionTag = false;
        $gotExtensionType = false;

        # Adds the optional parameters as params to the applet ("coordinates", "clusters", "standardDev", "randomSeed"
        #
        foreach ($args as $name => $value) {
            if (!$gotExtensionTag && $name == 'extensiontag') {
                $gotExtensionTag = true;
            }
            if (!$gotExtensionType && $name == 'extensiontype') {
                $gotExtensionType = true;
            }
            $output .= "  <param name='" . htmlspecialchars($name) . "' VALUE='" . htmlspecialchars($value) . "'>\n";
        }

        $cookieNames = array("UserName", "UserID", "_session");
        foreach($cookieNames as $ndx => $value) {
            $cookieKey = $wgCookiePrefix . $value;
            if (array_key_exists($cookieKey, $_COOKIE)) {
                $output .= "  <param name='" . htmlspecialchars($value) . "' VALUE='" . htmlspecialchars($_COOKIE[$cookieKey]) . "'>\n";
            }
        }

        # Send the cookie prefix as another parameter.
        $output .= "  <param name='cookiePrefix' VALUE='" . htmlspecialchars($wgCookiePrefix) . "'>\n";

        # If the extension tag hasn't been passed in a param yet, do it now with the default value.
        if (!$gotExtensionTag) {
            $output .= "  <param name='extensiontag' VALUE='clustering'>\n";
        }
        # If the extension type hasn't been passed in a param yet, do it now with the default of "tag".
        if (!$gotExtensionType) {
            $output .= "  <param name='extensiontype' VALUE='tag'>\n";
        }

        # Lastly, pass the extension tag used to enclose persisted applet data in the edit text for the page.
        $output .= "  <param name='dataTag' VALUE='clustering_data'>\n";

        $output .= "</applet>" .
          "<div id='clustering_demo_right_handle'></div>" .
          "<div id='clustering_demo_corner_handle'></div>" .
          "<div id='clustering_demo_bottom_handle'></div>" .
          "</div>\n";

        return $output;
    }

    # Returns an empty string.  Associate with a tag extension so that the extension can hold data in the edit text which you don't want to
    # affect the visible appearance of the page.
	function blankRender($input, $args, $parser) {
        return '';
    }

	# The renderer for the parser function, which simply emits in the format "<clustering ... />", so the
	# renderer for the tag extension clustering gets to handle it.
	#
    function clusteringDemoFunctionRender(&$parser, $coordinates = '', $clusters = '',
    	$randomSeed = '', $standardDev = '', $extensionTag = 'clustering', $extensionType = 'function') {

		# Just churn out the xml equivalent syntax, so clusteringDemoRender gets to parse it.
		# (Trying to return the html directly from here results in the "<", ">" being escaped.
		#
    	$output = "<clustering";
	    if ($coordinates != '') {
            $output .= " coordinates = \"$coordinates\"";
	    }
	    if ($clusters != '') {
            $output .= " clusters = \"$clusters\"";
	    }
	    if ($randomSeed != '') {
            $output .= " randomseed = \"$randomSeed\"";
	    }
	    if ($standardDev != '') {
            $output .= " standarddev = \"$standardDev\"";
	    }
        # The last 2 parameters are needed by the applet to figure out how to parse out the persisted data from its page.
        if ($extensionTag != '') {
            $output .= " extensiontag = \"$extensionTag\"";
        }
        if ($extensionType != '') {
            $output .= " extensiontype = \"$extensionType\"";
        }
	    $output .= " />";

        return $output;
    }
?>