package clustering;

/**
 * The class <code>ClusteringDemoSettings</code> encapsulates the data
 * shown in each row of the data shown in the class <code>ClusteringDemoPanel</code>,
 * which is the main panel of <code>DemoApplet</code>.
 * This is a very simple and immutable class, which is transformed to and from
 * XML using static methods in <code>ClusteringDemoSettingsFactory</code>
 * 
 * @author R. Scarberry
 *
 */
public class ClusteringDemoSettings {

    private int mCoordinates, mClusters;
    private double mStandardDev;
    private long mRandomSeed;
    private long mTimeStamp;
    private String mComment;
    
    /**
     * Fully-qualified constructor.
     * @param coordinates  number of coordinates.
     * @param clusters     number of clusters.
     * @param standardDev  the standard deviation.
     * @param randomSeed   the random seed for random number generation.
     * @param timeStamp    the timestamp, as the number of ms since Jan 1, 1970.
     * @param comment      a comment.
     */
    public ClusteringDemoSettings(int coordinates, int clusters,
    		double standardDev, long randomSeed, long timeStamp,
    		String comment) {
    	mCoordinates = coordinates;
    	mClusters = clusters;
    	mStandardDev = standardDev;
    	mRandomSeed = randomSeed;
    	mTimeStamp = timeStamp;
    	mComment = comment;
    }
    
    public int getCoordinates() {
    	return mCoordinates;
    }
    
    public int getClusters() {
    	return mClusters;
    }
    
    public double getStandardDev() {
    	return mStandardDev;
    }
    
    public long getRandomSeed() {
    	return mRandomSeed;
    }
    
    public long getTimeStamp() {
    	return mTimeStamp;
    }
    
    public String getComment() {
    	return mComment;
    }
    
    public int hashCode() {
    	int hc = (37 * mCoordinates) + mClusters;
    	hc = 37 * hc + (int)(mRandomSeed ^ (mRandomSeed >>> 32));
    	hc = 37 * hc + (int)(mTimeStamp ^ (mTimeStamp >>> 32));
    	long bits = Double.doubleToLongBits(mStandardDev);
    	hc = 37 * hc + (int)(bits ^ (bits >>> 32));
    	if (mComment != null) {
    		hc = 37 * hc + mComment.hashCode();
    	}
    	return hc;
    }
    
    public boolean equals(Object o) {
    	if (o == this) return true;
    	if (o instanceof ClusteringDemoSettings) {
    		ClusteringDemoSettings other = (ClusteringDemoSettings) o;
    		if (this.mTimeStamp == other.mTimeStamp && this.mCoordinates == other.mCoordinates && 
    				this.mRandomSeed == other.mRandomSeed && 
    				Double.doubleToLongBits(this.mStandardDev) == Double.doubleToLongBits(other.mStandardDev) && 
    				((this.mComment == null && other.mComment == null) || (this.mComment != null && this.mComment.equals(other.mComment)))) {
    			return true;
    		}
    	}
    	return false;
    }
}
