package clustering;

import java.util.*;
import java.text.*;
import javax.swing.*;
import javax.swing.table.*;

public class ClusteringDemoSettingsTableModel extends AbstractTableModel {

	private static final String[] COL_NAMES = { "Points", "Clusters", "Standard Dev", "Random Seed", "Time" };
	
	private java.util.List mSettingsList = new java.util.ArrayList();
	
    public static final String DATE_DISPLAY_FORMAT = "MMM d, yyyy HH:mm";

	private SimpleDateFormat mDateFormatter = new SimpleDateFormat(DATE_DISPLAY_FORMAT);

	public ClusteringDemoSettingsTableModel(ClusteringDemoSettings[] settings) {
		int n = settings != null ? settings.length : 0;
		for (int i=0; i<n; i++) {
			addSettings(settings[i]);
		}
	}
	
	public ClusteringDemoSettingsTableModel() {
	}
	
	public void addSettings(ClusteringDemoSettings settings) {
		mSettingsList.add(settings);
		int row = mSettingsList.size();
        super.fireTableRowsInserted(row, row);
	}
	
	public ClusteringDemoSettings getSettings(int rowIndex) {
		return (ClusteringDemoSettings) mSettingsList.get(rowIndex);
	}
	
	public void setData(ClusteringDemoSettings[] settings) {
		mSettingsList.clear();
		int n = settings != null ? settings.length : 0;
		for (int i=0; i<n; i++) {
			mSettingsList.add(settings[i]);
		}
		super.fireTableDataChanged();
	}
	
	public ClusteringDemoSettings[] getData() {
		int sz = mSettingsList.size();
		ClusteringDemoSettings[] all = new ClusteringDemoSettings[sz];
		if (sz > 0) {
			mSettingsList.toArray(all);
		}
		return all;
	}
	
	public ClusteringDemoSettings findSettings(int coordinates, int clusters,
			double standardDev, long randomSeed) {
		Iterator it = mSettingsList.iterator();
		while(it.hasNext()) {
			ClusteringDemoSettings settings = (ClusteringDemoSettings) it.next();
			if (settings.getCoordinates() == coordinates &&
					settings.getClusters() == clusters &&
					settings.getStandardDev() == standardDev &&
					settings.getRandomSeed() == randomSeed) {
				return settings;
			}
		}
		return null;
	}
	
	public String getColumnName(int column) {
		return COL_NAMES[column];
	}
	
	public Class getColumnClass() {
		return String.class;
	}
	
	public int getColumnCount() {
		return 5;
	}

	public int getRowCount() {
		return mSettingsList.size();
	}

	public Object getValueAt(int rowIndex, int columnIndex) {
		ClusteringDemoSettings settings = (ClusteringDemoSettings) mSettingsList.get(rowIndex);
		switch(columnIndex) {
		case 0:
			return String.valueOf(settings.getCoordinates());
		case 1:
			return String.valueOf(settings.getClusters());
		case 2:
			return String.valueOf(settings.getStandardDev());
		case 3:
			return String.valueOf(settings.getRandomSeed());
		case 4:
			return formatTimeStamp(settings.getTimeStamp());
		}
		throw new IndexOutOfBoundsException("invalid column: " + columnIndex);
	}
	
	public String formatTimeStamp(long ts) {
		return mDateFormatter.format(new Date(ts));
	}
	
	public String getComment(int rowIndex) {
		ClusteringDemoSettings settings = (ClusteringDemoSettings) mSettingsList.get(rowIndex);
		return settings.getComment();
	}

}
