package clustering;

import java.util.ArrayList;
import java.util.List;

import org.jfree.data.DomainOrder;
import org.jfree.data.general.AbstractSeriesDataset;
import org.jfree.data.xy.IntervalXYDataset;
import org.jfree.data.xy.IntervalXYDelegate;

public class XYDataset extends AbstractSeriesDataset 
implements IntervalXYDataset {

	private static final long serialVersionUID = 3881054018765280507L;

	private List mSeriesList = new ArrayList();
    private IntervalXYDelegate mIntervalDelegate = new IntervalXYDelegate(this, false);
    
    public XYDataset() {}
    
    public void addData(Comparable key, double[][] data, int[] indexes) {
        if (key == null || data == null) {
            throw new NullPointerException();
        }
        mSeriesList.add(new Series(key, data, indexes));
        fireDatasetChanged();
    }
    
    public void clear() {
        if (getSeriesCount() > 0) {
            mSeriesList.clear();
            fireDatasetChanged();
        }
    }
    
    public int getSeriesCount() {
        return mSeriesList.size();
    }

    public Comparable getSeriesKey(int series) {
        return ((Series) mSeriesList.get(series)).getKey();
    }

    public Number getEndX(int series, int item) {
        return mIntervalDelegate.getEndX(series, item);
    }

    public double getEndXValue(int series, int item) {
        return mIntervalDelegate.getEndXValue(series, item);
    }

    public Number getEndY(int series, int item) {
        return this.getY(series, item);
    }

    public double getEndYValue(int series, int item) {
        return this.getYValue(series, item);
    }

    public Number getStartX(int series, int item) {
        return mIntervalDelegate.getStartX(series, item);
    }

    public double getStartXValue(int series, int item) {
        return mIntervalDelegate.getStartXValue(series, item);
    }

    public Number getStartY(int series, int item) {
        return this.getY(series, item);
    }

    public double getStartYValue(int series, int item) {
        return this.getYValue(series, item);
    }

    public DomainOrder getDomainOrder() {
        return DomainOrder.NONE;
    }

    public int getItemCount(int series) {
        return ((Series) mSeriesList.get(series)).size();
    }

    public Number getX(int series, int item) {
        return new Double(getXValue(series, item));
    }

    public double getXValue(int series, int item) {
        return ((Series) mSeriesList.get(series)).getXValue(item);
    }

    public Number getY(int series, int item) {
        return new Double(getYValue(series, item));
    }

    public double getYValue(int series, int item) {
        return ((Series) mSeriesList.get(series)).getYValue(item);
    }

    private static class Series {
        
        private Comparable mKey;  // Typically a string
        private double[][] mData; // The data, N X 2 dimensionality.
        private int[] mIndexes;   // The indexes into data for this series.
    
        private Series (Comparable key, double[][] data, int[] indexes) {
            mKey = key;
            mData = data;
            // Defensively copy the indexes.
            int n = indexes != null ? indexes.length : 0;
            mIndexes = new int[n];
            if (n > 0) {
                System.arraycopy(indexes, 0, mIndexes, 0, n);
            }
        }
        
        private int size() {
            return mIndexes.length;
        }
        
        private Comparable getKey() {
            return mKey;
        }
        
        private double getXValue(int item) {
            return mData[mIndexes[item]][0];
        }
        
        private double getYValue(int item) {
            return mData[mIndexes[item]][1];
        }
    }
}
